from pathlib import Path
from typing import Any, Dict

import yaml


class ConfigManager:
    """Configuration manager class. Manages the configuration of Yoda."""

    def __init__(self):
        self.config_file = self.get_default_config_file()
        self.config: Dict[str, Any] = dict()
        self.load()

    def load(self):
        """Load the configuration from the configuration file."""
        if self.config_file.exists():
            with open(self.config_file) as f:
                self.config = yaml.safe_load(f)
        else:
            self.config = self.get_default_config()

    def save(self):
        """Save the configuration to the configuration file."""
        self.config_file.parent.mkdir(parents=True, exist_ok=True)
        with open(self.config_file, "w") as f:
            yaml.safe_dump(self.config, f)

    def get(self, key: str, default: Any = None) -> Any:
        """Get a configuration value by key."""
        return self.config.get(key, default)

    def set(self, key: str, value: Any):
        """Set a configuration value by key."""
        self.config[key] = value
        self.save()

    def get_yoda_config_dir(self) -> Path:
        return Path.home() / ".yoda"

    def get_yoda_plugins_dir(self) -> Path:
        return self.get_yoda_config_dir() / "plugins"

    def get_default_config_file(self) -> Path:
        return self.get_yoda_config_dir() / "config.yaml"

    def get_default_config(self) -> Dict[str, Any]:
        return {
            "user": "Skywalker",
            "plugins": {},
        }



import typer

from yodapa.plugin_manager.decorator import yoda_plugin


@yoda_plugin(name="bye")
class ByePlugin:
    """
    Bye plugin. Say goodbye.

    Example:
        $ yoda bye goodbye --name MP
        $ yoda bye goodbye
    """

    def goodbye(self, name: str = None):
        """Say goodbye."""
        name = name or "Padawan"
        typer.echo(f"Goodbye {name}!")



import importlib
import inspect
import pkgutil
from typing import List

import typer

from yodapa.config import ConfigManager


class PluginManager:
    """Plugin manager class. Manages the plugins in the 'plugins' directory and the local plugins directory."""

    def __init__(self, app: typer.Typer, config: ConfigManager):
        self.app: typer.Typer = app
        self.config: ConfigManager = config
        self.plugins: List = []

    def discover_plugins(self):
        """Discover plugins in the 'plugins' directory and the local plugins directory."""

        # 1. Discover plugins within the 'plugins' directory
        plugins_pkg = importlib.import_module('yodapa.plugins')
        plugins_path = plugins_pkg.__path__

        for finder, name, ispkg in pkgutil.iter_modules(plugins_path):
            # print("finder", finder, "name", name, "ispkg", ispkg)
            try:
                module = importlib.import_module(f'yodapa.plugins.{name}')
                for attribute_name in dir(module):
                    plugin_class = getattr(module, attribute_name)
                    if inspect.isclass(plugin_class) and hasattr(plugin_class(), "typer_app"):
                        plugin_instance = plugin_class()
                        self.plugins.append(plugin_instance)
            except Exception as e:
                typer.echo(f"Failed to load plugin {name}: {e}", err=True)

        # 2. Discover plugins in the local plugins directory
        local_plugins_dir = self.config.get_yoda_plugins_dir()
        if local_plugins_dir.exists() and local_plugins_dir.is_dir():
            for finder, name, ispkg in pkgutil.iter_modules([str(local_plugins_dir)]):
                # print("finder", finder, "name", name, "ispkg", ispkg)
                try:
                    module_path = local_plugins_dir / f"{name}.py"
                    if not module_path.exists():
                        typer.echo(f"Plugin module {name} does not exist at {module_path}", err=True)
                        continue

                    # Load the module from the file path
                    spec = importlib.util.spec_from_file_location(name, str(module_path))
                    if spec is None:
                        typer.echo(f"Could not load spec for module {name}", err=True)
                        continue

                    module = importlib.util.module_from_spec(spec)
                    spec.loader.exec_module(module)

                    # typer.echo(f"Imported module: {module.__name__}")

                    # Find the plugin class in the module (same as above)
                    for attribute_name in dir(module):
                        plugin_class = getattr(module, attribute_name)
                        if inspect.isclass(plugin_class) and hasattr(plugin_class(), "typer_app"):
                            plugin_instance = plugin_class()
                            self.plugins.append(plugin_instance)
                except Exception as e:
                    typer.echo(f"Failed to load local plugin {name}: {e}", err=True)

        # uncomment to debug
        # print("Plugins discovered:", self.plugins)

    def load_plugins(self):
        """Load the plugins into the yoda typer app."""
        for plugin in self.plugins:
            try:
                self.app.add_typer(plugin.typer_app, name=plugin.name, help=f"{plugin.name} plugin commands")
                # typer.echo(f"Loaded plugin: {plugin.name}")
            except Exception as e:
                typer.echo(f"Error loading plugin {plugin.name}: {e}", err=True)

    def enable_plugin(self, plugin_name: str):
        # TODO: implement
        pass

    def disable_plugin(self, plugin_name: str):
        # TODO: implement
        pass



import inspect
from typing import Annotated, Optional

import typer


def yoda_plugin(name: Annotated[Optional[str], typer.Argument()] = None):
    """
    Decorator to turn a class into a Yoda PA plugin.
    All public methods of the class are added as Typer commands.
    """

    def decorator(cls):
        nonlocal name
        name = name or cls.__name__.lower()

        def __init__(self):
            self.typer_app = typer.Typer(name=name, help=f"{name} plugin commands")

            for method_name, method in inspect.getmembers(self, predicate=inspect.ismethod):
                # Skip private methods
                if method_name.startswith("_"):
                    continue

                self.typer_app.command()(method)

        cls.__init__ = __init__
        cls.name = name
        return cls

    return decorator



import typer

from yodapa.plugin_manager.decorator import yoda_plugin


@yoda_plugin(name="hi")
class HiPlugin:
    """
    Hi plugin. Say hello.

    Example:
        $ yoda hi hello --name MP
        $ yoda hi hello
    """

    def hello(self, name: str = None):
        """Say hello."""
        name = name or "Padawan"
        typer.echo(f"Hello {name}!")

    def _private_method_should_not_be_added(self):
        """This method should not be added as a command."""
        raise NotImplementedError()
